@@ -11,7 +11,8 @@ distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License."""
-from time import time
+import csv
+from time import time, strftime, localtime
 from datetime import datetime, timedelta
 from random import shuffle
 from httplib import CannotSendRequest
@@ -41,6 +42,12 @@ def renderView(request):
   start = time()
   (graphOptions, requestOptions) = parseOptions(request)
   useCache = 'noCache' not in requestOptions
+  requestContext = {
+    'startTime' : requestOptions['startTime'],
+    'endTime' : requestOptions['endTime'],
+    'data' : []
+  }
+  data = requestContext['data']
 
   # First we check the request cache
   if useCache:
@@ -55,7 +62,6 @@ def renderView(request):
 
   # Now we prepare the requested data
   if requestOptions['graphType'] == 'pie':
-    data = []
     for target in requestOptions['targets']:
       if target.find(':') >= 0:
         try:
@@ -65,12 +71,11 @@ def renderView(request):
           raise ValueError, "Invalid target '%s'" % target
         data.append( (name,value) )
       else:
-        timeInterval = (requestOptions['startTime'], requestOptions['endTime'])
-        seriesList = evaluateTarget(target, timeInterval)
+        seriesList = evaluateTarget(requestContext, target)
 
         for series in seriesList:
           func = PieFunctions[requestOptions['pieMode']]
-          data.append( (series.name, func(series) or 0 ))
+          data.append( (series.name, func(requestContext, series) or 0 ))
 
   elif requestOptions['graphType'] == 'line':
     # Let's see if at least our data is cached
@@ -88,13 +93,11 @@ def renderView(request):
       cachedData = None
 
     if cachedData is not None:
-      data = cachedData
+      requestContext['data'] = data = cachedData
     else: # Have to actually retrieve the data now
-      data = []
-      timeInterval = (requestOptions['startTime'], requestOptions['endTime'])
       for target in requestOptions['targets']:
         t = time()
-        seriesList = evaluateTarget(target, timeInterval)
+        seriesList = evaluateTarget(requestContext, target)
         log.rendering("Retrieval of %s took %.6f" % (target, time() - t))
         data.extend(seriesList)
 
@@ -110,6 +113,17 @@ def renderView(request):
       log.rendering('Total pickle rendering time %.6f' % (time() - start))
       return response
 
+    if requestOptions.get('format') == 'csv':
+      response = HttpResponse(mimetype='text/csv')
+      writer = csv.writer(response, dialect='excel')
+
+      for series in data:
+        for i, value in enumerate(series):
+          timestamp = localtime( series.start + (i * series.step) )
+          writer.writerow( (series.name, strftime("%Y-%m-%d %H:%M:%S", timestamp), value) )
+
+      return response
+
     if 'rawData' in requestOptions:
       response = HttpResponse(mimetype='text/plain')
       for series in data:
@@ -161,6 +175,8 @@ def parseOptions(request):
     requestOptions['pickle'] = True
   if 'rawData' in queryParams:
     requestOptions['rawData'] = True
+  if 'format' in queryParams:
+    requestOptions['format'] = queryParams['format']
   if 'noCache' in queryParams:
     requestOptions['noCache'] = True
 
@@ -173,7 +189,7 @@ def parseOptions(request):
       elif '.' in val and (val.replace('.','',1).isdigit() or (val.startswith('-') and val[1:].replace('.','',1).isdigit())):
         val = float(val)
       elif val.lower() in ('true','false'):
-        val = eval( val.lower().capitalize() )
+        val = val.lower() == 'true'
       elif val.lower() == 'default' or val == '':
         continue
       graphOptions[opt] = val
